use anyhow::Result;
use wasmtime::*;

const PAGE: usize = 65536;

fn main() -> Result<()> {
    let mut config = Config::new();
    let engine = Engine::new(&config).unwrap();
    let bytes = include_bytes!("../tinygo-gc-debug.wasm");
    let module = Module::new(&engine, bytes)?;

    // Create a `Linker` which will be later used to instantiate this module.
    // Host functionality is defined by name within the `Linker`.
    let mut linker = Linker::new(&engine);
    linker.func_wrap(
        "wasi_snapshot_preview1",
        "fd_write",
        |caller: Caller<'_, StoreLimits>, a: i32, b: i32, c: i32, d: i32| -> i32 { 0 },
    )?;
    linker.func_wrap(
        "fs",
        "load_rom_file",
        |caller: Caller<'_, StoreLimits>, a: i32, b: i32| {},
    )?;
    linker.func_wrap(
        "graphics",
        "draw",
        |caller: Caller<'_, StoreLimits>, ptr: u32, len: u32| {},
    )?;

    let limits = StoreLimitsBuilder::new().memory_size(2 * PAGE).build();
    let mut store = Store::new(&engine, limits);
    store.limiter(|limits| limits);
    let instance = linker.instantiate(&mut store, &module)?;

    let boot = instance.get_typed_func::<(), ()>(&mut store, "boot")?;
    boot.call(&mut store, ())?;

    let update = instance.get_typed_func::<(), ()>(&mut store, "update")?;
    update.call(&mut store, ())?;

    let mem = instance.get_memory(&mut store, "memory").unwrap();
    std::fs::write("memory.bin", mem.data(&mut store))?;

    Ok(())
}
